unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, DirTree, TreeScan, Menus;

type
  TMainForm = class(TForm)
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    CheckBox1: TCheckBox;
    ComboBox1: TComboBox;
    ListBox2: TListBox;
    Button1: TButton;
    ComboBox2: TComboBox;
    Button2: TButton;
    SaveDialog: TSaveDialog;
    Panel2: TPanel;
    DirTree: TDirTree;
    ListBox1: TListBox;
    TreeScanner: TTreeScanner;
    CheckBox2: TCheckBox;
    Timer1: TTimer;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    PopupMenu1: TPopupMenu;
    Clear1: TMenuItem;
    Delete1: TMenuItem;
    N1: TMenuItem;
    Transfer1: TMenuItem;
    Save1: TMenuItem;
    SaveSel: TMenuItem;
    Selectall1: TMenuItem;
    PopupMenu2: TPopupMenu;
    Selectall2: TMenuItem;
    Selectnone1: TMenuItem;
    Selectnone2: TMenuItem;
    Transfer2: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure About(Sender: TObject);
    procedure DirTreeChange(Sender: TObject; Node: TTreeNode);
    procedure TreeScannerFileFound(Sender: TObject);
    procedure ReScan(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure TreeScannerScanFinished(Sender: TObject);
    procedure KeyDown1(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Timer1Timer(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure Selectall1Click(Sender: TObject);
    procedure SaveSelClick(Sender: TObject);
    procedure Selectall2Click(Sender: TObject);
    procedure Selectnone1Click(Sender: TObject);
    procedure Selectnone2Click(Sender: TObject);
  private
    { Private declarations }
  public
    procedure StartScan;
    procedure StopScan;
  end;

var
  MainForm: TMainForm;

implementation
Uses AboutFrm;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  Timer1.Interval   := 10;
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
{  if Length(Application.Hint) > 0 then
    StatusBar1.Panels[0].Text := Application.Hint
  else
}
    begin
      StatusBar1.Panels[0].Text := Format('%d file(s) found.',[ListBox1.Items.Count]);
      StatusBar1.Panels[1].Text := Format('%d file(s) transfered.',[ListBox2.Items.Count]);
    end;
end;

procedure TMainForm.About(Sender: TObject);
begin
  with TAboutDlg.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
begin
  Timer1.Free;
  DirTree.InitialDirectory := ExtractFileDir(Application.Name);
  StartScan;
end;

procedure TMainForm.StartScan;
begin
  StatusBar1.Panels[0].Text := Format('Scanning %s',[ExtractFileDir(DirTree.Directory)]);
  ListBox1.Clear;
  TreeScanner.InitialDirectory := DirTree.Directory;
  TreeScanner.Recursive := CheckBox1.Checked;
  TreeScanner.FileMask  := ComboBox1.Text;
  TreeScanner.ScanTree;
end;

procedure TMainForm.StopScan;
begin
  TreeScanner.StopScan;
end;

procedure TMainForm.ReScan(Sender: TObject);
begin
  StopScan;
  StartScan;
end;

procedure TMainForm.DirTreeChange(Sender: TObject; Node: TTreeNode);
begin
  ReScan(Sender);
end;

procedure TMainForm.TreeScannerFileFound(Sender: TObject);
begin
  Application.ProcessMessages;
  if CheckBox2.Checked then ListBox1.Items.Add(ExtractRelativePath(TreeScanner.InitialDirectory, TreeScanner.FileFound))
                       else ListBox1.Items.Add(TreeScanner.FileFound);
end;

procedure TMainForm.Button1Click(Sender: TObject);

Function LastPos(SearchStr, InStr: String): Integer;
var p,q: Integer;
    s: string;
begin
  s := Instr;
  p := 0;
  q := 0;
  repeat
    Inc(q,p);
    LastPos := q;
    s := Copy(s, p+1, 255);
    p := Pos(SearchStr, s);
  until (p = 0) {or (Length(s) = 0)};
end;

function ExtractShortFileName(FileName:String): String;
var S: String;
    P: Integer;
begin
  S := ExtractFileName(FileName);
  p := LastPos('.',S);
  if P = 0 then ExtractShortFileName := S
           else ExtractShortFileName := Copy(S, 1, p-1);
end;

Function FormatString(FormatStr, SubStr, ReplaceStr: String):String;
var p : Integer;
    S : String;
begin
  S := FormatStr;
  p := Pos(SubStr, S);
  if p > 0 then
  begin
    Delete(S, p, Length(SubStr));
    Insert(ReplaceStr, S, p);
  end;
  FormatString := S;
end;

Function GetString(FName: String):String;
Var s: String;
begin
  S :=ComboBox2.Text;
  S :=FormatString(S, '%FName',     ExtractShortFileName(FName));
  S :=FormatString(S, '%FExt',      ExtractFileExt(FName));
  S :=FormatString(S, '%FileName',  ExtractFileName(FName));
  S :=FormatString(S, '%File',      FName);
  S :=FormatString(S, '%Path',      ExtractFilePath(FName));
  S :=FormatString(S, '%Dir',       ExtractFileDir(FName));
  S :=FormatString(S, '%Drive',     ExtractFileDrive(FName));
  GetString := S;
end;

var
  i: Integer;
begin
  // Geen MultiSelect
  if ListBox1.SelCount = -1 then
  begin
  end else
  // MultiSelect; 0 = geen selectie dus alles else selected
  for i := 0 to ListBox1.Items.Count-1 do
   If (ListBox1.SelCount = 0) or ListBox1.Selected[i] then
      ListBox2.Items.Add(GetString(ListBox1.Items[i]));
  ShowHint(Sender);
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  if SaveDialog.Execute then
    ListBox2.Items.saveToFile(SaveDialog.FileName);
end;

procedure TMainForm.TreeScannerScanFinished(Sender: TObject);
begin
  ShowHint(Sender);
end;

procedure TMainForm.KeyDown1(Sender: TObject; var Key: Word; Shift: TShiftState);
var i: Integer;
begin
  if (Key=VK_F1) then About(Sender);
  if (Sender = ListBox1) and (Key = VK_Escape) then
   for i := 0 to ListBox1.Items.Count-1 do ListBox1.Selected[i] := False;
  if (Sender = ListBox2) and (Key = VK_Escape) then
   for i := 0 to ListBox2.Items.Count-1 do ListBox2.Selected[i] := False;
  if (Sender = ListBox2) and (Key = VK_DELETE) then
  begin
   for i := ListBox2.Items.Count-1 downto 0 do
    If ListBox2.Selected[i] then ListBox2.Items.Delete(i);
    ShowHint(Sender);
  end;
end;

procedure TMainForm.Selectall2Click(Sender: TObject);
var i: Integer;
begin
  for i := 0 to ListBox1.Items.Count-1 do
    ListBox1.Selected[i] := True;
end;

procedure TMainForm.Selectnone1Click(Sender: TObject);
var i: Integer;
begin
  for i := 0 to ListBox1.Items.Count-1 do
    ListBox1.Selected[i] := False;
end;

procedure TMainForm.Clear1Click(Sender: TObject);
begin
  ListBox2.Clear;
end;

procedure TMainForm.Delete1Click(Sender: TObject);
var i: Integer;
begin
  for i := ListBox2.Items.Count-1 downto 0 do
   If ListBox2.Selected[i] then ListBox2.Items.Delete(i);
  ShowHint(Sender);
end;

procedure TMainForm.Selectall1Click(Sender: TObject);
var i: Integer;
begin
   for i := 0 to ListBox2.Items.Count-1 do
    ListBox2.Selected[i] := True;
end;

procedure TMainForm.SaveSelClick(Sender: TObject);
var i   : Integer;
    List: TStringList;
begin
  List           := TStringList.Create;
  List.Duplicates:= dupAccept;
  List.Sorted    := False;
  List.Clear;
  for i := 0 to ListBox2.Items.Count-1 do
    If ListBox2.Selected[i] then List.Add(ListBox2.Items[i]);
  if SaveDialog.Execute then List.SaveToFile(SaveDialog.FileName);
  List.Free;
end;

procedure TMainForm.Selectnone2Click(Sender: TObject);
var i: Integer;
begin
  for i := 0 to ListBox2.Items.Count-1 do
    ListBox2.Selected[i] := False;
end;

end.
